import pymysql
import csv
import configparser
from google.cloud import secretmanager
from datetime import datetime
from airflow.models import Variable
import os


def extract_sql(configpath=None,
                hostname=None,
                database=None,
                table=None,
                in_airflow=True):

    parser = configparser.ConfigParser()
    parser.read(configpath)
    username = parser.get('mysql_config', 'username')
    secret_id = parser.get('mysql_config', 'secret_id')
    project_id = parser.get('gcp_config', 'project_id')

    client = secretmanager.SecretManagerServiceClient()
    name = f'projects/{project_id}/secrets/{secret_id}/versions/latest'
    response = client.access_secret_version(name=name)
    password = response.payload.data.decode('UTF-8')

    conn = pymysql.connect(host=hostname,
                           user=username,
                           password=password,
                           db=database,
                           port=3306)

    curr_date = datetime.now().strftime('%Y-%m-%d')
    filename = f'{table}-extract-{curr_date}.csv'
    data_folder = Variable.get('data_folder') if in_airflow else './'
    data_path = f'{data_folder}/{database}/{table}'

    if not os.path.exists(data_path):
        os.makedirs(data_path)

    with open(f'{data_path}/{filename}', 'w') as fp:
        csv_w = csv.writer(fp, delimiter='|')
        begin_time = datetime.now()
        batch = 200000
        checkpoint = 0
        print('starting loop')

        while True:
            loop_time = datetime.now()
            m_query = f"SELECT * FROM {table} ORDER BY id ASC LIMIT {checkpoint}, {batch}"
            m_cursor = conn.cursor()
            m_cursor.execute(m_query)
            results = m_cursor.fetchall()

            if checkpoint == 0:
                field_names = [i[0] for i in m_cursor.description]
                csv_w.writerow(field_names)

            if not results:
                print('breaking')
                break

            csv_w.writerows(results)
            checkpoint += batch
            print(f'checkpoint: {checkpoint}')

            query_time = datetime.now()
            time_taken = query_time - loop_time
            print(f'time taken to query: {time_taken}')

        write_time = datetime.now() - begin_time
        print(f'time taken to write: {write_time}')

    m_cursor.close()
    conn.close()



from google.cloud import storage
from airflow.models import Variable
from datetime import datetime


def load_gcs(project=None,
             database=None,
             table=None,
             gcs_bucket=None,
             processed=False,
             in_airflow=True):

    curr_date = datetime.now().strftime('%Y-%m-%d')

    if processed:
        filename = f'{table}-extract-{curr_date}-processed.csv'
        state = 'processed'
    else:
        filename = f'{table}-extract-{curr_date}.csv'
        state = 'raw'

    data_folder = Variable.get('data_folder') if in_airflow else './'
    data_path = f'{data_folder}/{database}/{table}'

    client = storage.Client(project=project)
    bucket = client.get_bucket(gcs_bucket)
    blob = bucket.blob(f'{state}/{database}/{table}/{filename}')
    blob.upload_from_filename(filename=f'{data_path}/{filename}')



from google.cloud import storage
from airflow.models import Variable
from datetime import datetime
import json
from dask import dataframe as dd


def create_dask_df(database=None,
                   table=None,
                   gcs_bucket=None,
                   in_airflow=True):

    curr_date = datetime.now().strftime('%Y-%m-%d')
    filename = f'{table}-extract-{curr_date}.csv'

    data_folder = Variable.get('data_folder') if in_airflow else './'
    data_path = f'{data_folder}/{database}/{table}/{filename}'

    storage_client = storage.Client()
    bucket = storage_client.bucket(gcs_bucket)
    blob = bucket.blob(f'schemas/{table}-dask-schema.json')
    dt_blob = blob.download_as_text()
    data_types = json.loads(dt_blob)

    ddf = dd.read_csv(data_path, delimiter='|', dtype=data_types)
    return ddf



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="orbit_etl",
    version="0.3.1",
    author="Benjamin Jeffrey",
    author_email="ben.jeffrey@orbitremit.com",
    description="Package for extracting from MySQL and loading to GCS",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/orbitremit/orbit_etl",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
    install_requires=[
        'pymysql',
        'google-cloud-secret-manager',
        'google-cloud-storage',
        'configparser',
        'dask',
        'dask[dataframe]'
    ]
)
